local _G = _G
local Button = CreateFrame("Button")
local Button_MT = {__index = Button}
Yata.Button = {}
Yata.Button.prototype = Button
Yata.Button.db = Yata.db
local LibKeyBound = LibStub("LibKeyBound-1.0")
local AceEvent = LibStub("AceEvent-3.0")
function Yata.Button:CreateBase(id, parent)
	local name = "YataTotemButton" .. id
	local button = setmetatable(CreateFrame("Button", name, parent, "SecureHandlerEnterLeaveTemplate, SecureHandlerAttributeTemplate, SecureActionButtonTemplate"), Button_MT)
	button.Id = id
	button.Parent = parent
	button.Name = name
	button:SetWidth(36)
	button:SetHeight(36)	
	button:RegisterForClicks("LeftButtonUp","RightButtonUp")		

	button.Icon = button:CreateTexture(name.."Icon")   
	button.Icon:SetAllPoints(button)
	
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	button:GetHighlightTexture():SetBlendMode("ADD")
	
	button.Cooldown = CreateFrame("Cooldown", name.."Cooldown",button,"CooldownFrameTemplate")
	button.Cooldown:SetWidth(36)
	button.Cooldown:SetHeight(36)
	button.Cooldown:SetPoint("CENTER",0,-1)
	
	button.HotKey = button:CreateFontString(name.."HotKey", "OVERLAY", "NumberFontNormalSmallGray")
	button.HotKey:SetWidth(36)
	button.HotKey:SetHeight(10)
	button.HotKey:SetJustifyH("RIGHT")
	button.HotKey:SetPoint("TOPLEFT",-2,-2)	
	
	button:SetNormalTexture("")	
	button.NormalTexture = button:GetNormalTexture()
	button.NormalTexture:SetWidth(66)
	button.NormalTexture:SetHeight(66)
	button.NormalTexture:ClearAllPoints()
	button.NormalTexture:SetPoint("CENTER", 0, -1)
	button.NormalTexture:Hide()
	button:SetAttribute("cooldown",0)

	button.TimerText = button:CreateFontString(name.."TimerText", "OVERLAY")
	button.TimerText:SetPoint("CENTER",0,-1)
	button.TimerText:SetFont("Fonts\\FRIZQT__.TTF", 11)
	button.TimerText:SetTextColor(0,1,0)
	button.TimerText:Show()
	
	button:RegisterEvents()
	button:UpdateHotkey()	
	return button
end

function Yata.Button:Create(id, parent, totem)
	local button = Yata.Button:CreateBase(id,parent)
	button.Totem = totem
	button:HookScript("OnEnter", function() button:OnEnter(button) end)
	button:HookScript("OnLeave", function() button:OnLeave() end)
	button:SetAttribute( "_onenter", [[  
		header:SetAttribute("show", true)
	]] )
	button:SetAttribute( "_onleave", [[  
		inHandler = header:IsUnderMouse(true)
		if not inHandler then			
			header:SetAttribute("show", false)
		end	    
	]] )
	button:SetAttribute( "_onattributechanged", [[
		if name == "newposition" then
			self:ClearAllPoints()
			self:SetAttribute("position", value)
			local orientation = header:GetAttribute("orientation")
			local anchor, sign
			if orientation == 1 then
				anchor = "BOTTOM"
				sign = 1
			elseif orientation == 2 then
				anchor = "TOP"
				sign = -1
			end
			self:SetPoint( anchor , header, anchor, 0, sign*value*(36 + header:GetAttribute("buttongap") or 0))
			if self:GetAttribute("position") < header:GetAttribute("visiblebuttons") or self:GetAttribute("cooldown") > 0 then
				self:Show()
			else
				self:Hide()
			end
		end
	]] )
	button:SetAttribute("totemcooldown",totem.Cooldown)
	return button		
end

function Yata.Button:CreateHeader(id, parent)
	local name = "YataElementHeader" .. id
	local header = CreateFrame("Frame", name, parent, "SecureHandlerAttributeTemplate, SecureHandlerEnterLeaveTemplate")
	header.Name = name
	header.Parent = parent;
	header:SetWidth(36)
	header:SetHeight(0)	
	--[[
	header.Texture = header:CreateTexture()
	header.Texture:SetTexture(0, 0, 0.5, 1)
	header.Texture:SetAllPoints(header)	
	--]]
	header:SetAttribute("_onattributechanged", [[
		if name == "show" then
			local buttongap = self:GetAttribute("buttongap") or 0
			local headerSize = 36
			local visibleButtons = self:GetAttribute("visiblebuttons") or 1
			buttons = newtable(self:GetChildren())
			if value == true then
				for k,v in ipairs(buttons) do 
					v:Show()
				end
				headerSize = (36 + buttongap) * (#buttons - 1) + 36
			else
				for k,v in ipairs(buttons) do 
					if v:GetAttribute("position") < visibleButtons or v:GetAttribute("cooldown") > 0 then
						v:Show()
					else
						v:Hide()
					end
				end
			end
			self:SetHeight(headerSize)
		end
	]] )
	return header
end

function Button:UpdateCooldown()
	local spellId
	if Yata.db.profile.ButtonCooldown then
		spellId = self.Totem.SpellID
	else
		totem = Yata.TotemData[GetSpellInfo(8071)] -- Stoneskin Totem
		spellId = totem[#totem].SpellID
	end
	local start, duration, enable = GetSpellCooldown(spellId, BOOKTYPE_SPELL)
	CooldownFrame_SetTimer(self.Cooldown, start, duration, enable)
end

function Button:SetOnClickScript(autobutton)
	if autobutton then
		self:UnwrapScript(self,"OnClick")
		self:WrapScript(self,"OnClick","return nil, true", [=[
			pos = self:GetAttribute("position")
			if pos > 0 then
				local buttonSize = header:GetAttribute("buttongap") + 36
				children = newtable(header:GetChildren())
				for i,b in ipairs(children) do			
					local oldpos = b:GetAttribute("position")
					local newpos = oldpos
					if oldpos < pos then
						newpos = oldpos + 1
					elseif oldpos == pos then
						newpos = 0
					end
					b:SetAttribute("newposition",newpos)
				end
			end
			local cd = self:GetAttribute("totemcooldown")
			if cd > 0 then
				self:SetAttribute("cooldown",cd)
				control:SetTimer(cd, nil, [[
					self:SetAttribute("cooldown",0) 
					if not header:IsUnderMouse(true) then 
						header:SetAttribute("show",false)  
						end
				]] )
			end
			header:SetAttribute("show",false)
		]=] )
	else
		self:UnwrapScript(self,"OnClick")
		self:WrapScript(self,"OnClick","return nil, true", [[header:SetAttribute("show",false)]])
	end	
end

function Button:RegisterEvents()
	AceEvent.RegisterEvent(self,"UPDATE_BINDINGS")
end

function Button:UPDATE_BINDINGS()
	self:UpdateHotkey()
end
function Button:OnEnter(self)
	if Yata.db.profile.ShowTooltip then
		self:SetTooltip()
	end	
	LibKeyBound:Set(self)
end

function Button:OnLeave()
	GameTooltip:Hide()
end

function Button:GetHotkey()
	local key1 = GetBindingKey(format("CLICK %s:LeftButton", self:GetName()))
	local displayKey = LibKeyBound:ToShortKey(key1)
	return displayKey
end

function Button:UpdateHotkey()
	local key = self:GetHotkey()
	local hotkey = self.HotKey
	if key then
		hotkey:SetText(key)
		hotkey:Show()
	end
end

function Button:SetTooltip()
	if  GetCVar("UberTooltips") == "1"  then
		GameTooltip_SetDefaultAnchor(GameTooltip, self)
	else
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	end
	GameTooltip:SetText(self.Totem.Name, 1.0, 1.0, 1.0)
	--[[
	if Yata.db.profile.ShortTooltip then
	else
		GameTooltip:SetSpell( self.Totem.SpellID, BOOKTYPE_SPELL )
	end
	--]]
end
function Button:GetActionName()
	return self.Totem.Name
end